@@ -5,18 +5,16 @@ See: https://github.com/pytest-dev/pytest/issues/3730#issuecomment-567142496
 
 
 def pytest_configure(config):
-    config.addinivalue_line(
-        "markers", "deselect_if(func): function to deselect tests from parametrization"
-    )
+    config.addinivalue_line("markers", "deselect_if(func): function to deselect tests from parametrization")
 
 
 def pytest_collection_modifyitems(config, items):
     removed = []
     kept = []
     for item in items:
-        m = item.get_closest_marker('deselect_if')
+        m = item.get_closest_marker("deselect_if")
         if m:
-            func = m.kwargs['func']
+            func = m.kwargs["func"]
             if func(**item.callspec.params):
                 removed.append(item)
                 continue
