@@ -1,10 +1,10 @@
-from ._version import get_versions
 from .aggregate_purepy import aggregate as aggregate_py
 
 
 def dummy_no_impl(*args, **kwargs):
-    raise NotImplementedError("You may need to install another package (numpy, "
-                              "weave, or numba) to access a working implementation.")
+    raise NotImplementedError(
+        "You may need to install another package (numpy or numba) to access a working implementation."
+    )
 
 
 aggregate = aggregate_py
@@ -16,22 +16,16 @@ except ImportError:
     multi_arange = multi_cumsum = label_contiguous_1d = dummy_no_impl
 else:
     from .aggregate_numpy import aggregate
+
     aggregate_np = aggregate
     from .aggregate_numpy_ufunc import aggregate as aggregate_ufunc
-    from .utils_numpy import (label_contiguous_1d, multi_arange, relabel_groups_masked,
-                              relabel_groups_unique, unpack)
-
-
-try:
-    try:
-        import weave
-    except ImportError:
-        from scipy import weave
-except ImportError:
-    aggregate_wv = None
-else:
-    from .aggregate_weave import aggregate as aggregate_wv, step_count, step_indices
-    aggregate = aggregate_wv
+    from .utils import (
+        label_contiguous_1d,
+        multi_arange,
+        relabel_groups_masked,
+        relabel_groups_unique,
+        unpack,
+    )
 
 
 try:
@@ -39,7 +33,9 @@ try:
 except ImportError:
     aggregate_nb = None
 else:
-    from .aggregate_numba import aggregate as aggregate_nb, step_count, step_indices
+    from .aggregate_numba import aggregate as aggregate_nb
+    from .aggregate_numba import step_count, step_indices
+
     aggregate = aggregate_nb
 
 
@@ -47,5 +43,14 @@ def uaggregate(group_idx, a, **kwargs):
     return unpack(group_idx, aggregate(group_idx, a, **kwargs))
 
 
-__version__ = get_versions()['version']
-del get_versions
+try:
+    # Version is added only when packaged
+    from ._version import __version__
+except ImportError:
+    try:
+        from setuptools_scm import get_version
+    except ImportError:
+        __version__ = "0.0.0"
+    else:
+        __version__ = get_version(root="..", relative_to=__file__)
+        del get_version
