@@ -1,10 +1,15 @@
-from __future__ import division
-
 import numba as nb
 import numpy as np
 
-from .utils import aggregate_common_doc, funcs_no_separate_nan, get_func, isstr
-from .utils_numpy import aliasing, check_dtype, check_fill_value, input_validation
+from .utils import (
+    aggregate_common_doc,
+    aliasing,
+    check_dtype,
+    check_fill_value,
+    funcs_no_separate_nan,
+    get_func,
+    input_validation,
+)
 
 
 class AggregateOp(object):
@@ -37,9 +42,26 @@ class AggregateOp(object):
         self._jit_scalar = self.callable(self.nans, self.reverse, scalar=True)
         self._jit_non_scalar = self.callable(self.nans, self.reverse, scalar=False)
 
-    def __call__(self, group_idx, a, size=None, fill_value=0, order='C',
-                 dtype=None, axis=None, ddof=0):
-        iv = input_validation(group_idx, a, size=size, order=order, axis=axis, check_bounds=False, func=self.func)
+    def __call__(
+        self,
+        group_idx,
+        a,
+        size=None,
+        fill_value=0,
+        order="C",
+        dtype=None,
+        axis=None,
+        ddof=0,
+    ):
+        iv = input_validation(
+            group_idx,
+            a,
+            size=size,
+            order=order,
+            axis=axis,
+            check_bounds=False,
+            func=self.func,
+        )
         group_idx, a, flat_size, ndim_idx, size, unravel_shape = iv
 
         # TODO: The typecheck should be done by the class itself, not by check_dtype
@@ -99,7 +121,7 @@ class AggregateOp(object):
 
     @classmethod
     def callable(cls, nans=False, reverse=False, scalar=False):
-        """ Compile a jitted function doing the hard part of the job """
+        """Compile a jitted function doing the hard part of the job"""
         _valgetter = cls._valgetter_scalar if scalar else cls._valgetter
         valgetter = nb.njit(_valgetter)
         outersetter = nb.njit(cls._outersetter)
@@ -155,11 +177,12 @@ class AggregateOp(object):
 
 class Aggregate2pass(AggregateOp):
     """Base class for everything that needs to process the data twice like mean, var and std."""
+
     @classmethod
     def callable(cls, nans=False, reverse=False, scalar=False):
         # Careful, cls needs to be passed, so that the overwritten methods remain available in
         # AggregateOp.callable
-        loop_1st = super(Aggregate2pass, cls).callable(nans=nans, reverse=reverse, scalar=scalar)
+        loop_1st = super().callable(nans=nans, reverse=reverse, scalar=scalar)
 
         _2pass_inner = nb.njit(cls._2pass_inner)
 
@@ -190,6 +213,7 @@ class Aggregate2pass(AggregateOp):
 
 class AggregateNtoN(AggregateOp):
     """Base class for cumulative functions, where the output size matches the input size."""
+
     outer = True
 
     @staticmethod
@@ -199,6 +223,7 @@ class AggregateNtoN(AggregateOp):
 
 class AggregateGeneric(AggregateOp):
     """Base class for jitting arbitrary functions."""
+
     counter_fill_value = None
 
     def __init__(self, func, **kwargs):
@@ -206,8 +231,17 @@ class AggregateGeneric(AggregateOp):
         self.__dict__.update(kwargs)
         self._jitfunc = self.callable(self.nans)
 
-    def __call__(self, group_idx, a, size=None, fill_value=0, order='C',
-                 dtype=None, axis=None, ddof=0):
+    def __call__(
+        self,
+        group_idx,
+        a,
+        size=None,
+        fill_value=0,
+        order="C",
+        dtype=None,
+        axis=None,
+        ddof=0,
+    ):
         iv = input_validation(group_idx, a, size=size, order=order, axis=axis, check_bounds=False)
         group_idx, a, flat_size, ndim_idx, size, _ = iv
 
@@ -218,7 +252,7 @@ class AggregateGeneric(AggregateOp):
         ret, _, _, _ = self._initialize(flat_size, fill_value, dtype, input_dtype, group_idx.size)
         group_idx = np.ascontiguousarray(group_idx)
 
-        sortidx = np.argsort(group_idx, kind='mergesort')
+        sortidx = np.argsort(group_idx, kind="mergesort")
         self._jitfunc(sortidx, group_idx, a, ret)
 
         # Deal with ndimensional indexing
@@ -460,13 +494,33 @@ class CumMin(AggregateNtoN, Min):
 
 def get_funcs():
     funcs = dict()
-    for op in (Sum, Prod, Len, All, Any, Last, First, AllNan, AnyNan, Min, Max,
-               ArgMin, ArgMax, Mean, Std, Var, SumOfSquares,
-               CumSum, CumProd, CumMax, CumMin):
+    for op in (
+        Sum,
+        Prod,
+        Len,
+        All,
+        Any,
+        Last,
+        First,
+        AllNan,
+        AnyNan,
+        Min,
+        Max,
+        ArgMin,
+        ArgMax,
+        Mean,
+        Std,
+        Var,
+        SumOfSquares,
+        CumSum,
+        CumProd,
+        CumMax,
+        CumMin,
+    ):
         funcname = op.__name__.lower()
         funcs[funcname] = op(funcname)
         if funcname not in funcs_no_separate_nan:
-            funcname = 'nan' + funcname
+            funcname = "nan" + funcname
             funcs[funcname] = op(funcname, nans=True)
     return funcs
 
@@ -475,10 +529,11 @@ _impl_dict = get_funcs()
 _default_cache = {}
 
 
-def aggregate(group_idx, a, func='sum', size=None, fill_value=0, order='C',
-              dtype=None, axis=None, cache=True, **kwargs):
+def aggregate(
+    group_idx, a, func="sum", size=None, fill_value=0, order="C", dtype=None, axis=None, cache=True, **kwargs
+):
     func = get_func(func, aliasing, _impl_dict)
-    if not isstr(func):
+    if not isinstance(func, str):
         if cache in (None, False):
             # Keep None and False in order to accept empty dictionaries
             aggregate_op = AggregateGeneric(func)
@@ -492,9 +547,12 @@ def aggregate(group_idx, a, func='sum', size=None, fill_value=0, order='C',
         return func(group_idx, a, size, fill_value, order, dtype, axis, **kwargs)
 
 
-aggregate.__doc__ = """
+aggregate.__doc__ = (
+    """
     This is the numba implementation of aggregate.
-    """ + aggregate_common_doc
+    """
+    + aggregate_common_doc
+)
 
 
 @nb.njit
