@@ -4,43 +4,46 @@ results are compared against the results of the other implementations. Implement
 may throw NotImplementedError in order to show missing functionality without throwing
 test errors. 
 """
-import sys
 from itertools import product
+
 import numpy as np
 import pytest
 
-from . import (aggregate_purepy, aggregate_numpy_ufunc, aggregate_numpy,
-               aggregate_weave, aggregate_numba, aggregate_pandas,
-               _wrap_notimplemented_xfail, _impl_name, func_list)
+from . import (
+    _impl_name,
+    _wrap_notimplemented_skip,
+    aggregate_numba,
+    aggregate_numpy,
+    aggregate_numpy_ufunc,
+    aggregate_pandas,
+    aggregate_purepy,
+    func_list,
+)
 
 
 class AttrDict(dict):
     __getattr__ = dict.__getitem__
 
 
-TEST_PAIRS = ['np/py', 'ufunc/np', 'numba/np', 'pandas/np']
-if sys.version_info.major == 2:
-    TEST_PAIRS.append('weave/np')
+TEST_PAIRS = ["np/py", "ufunc/np", "numba/np", "pandas/np"]
 
 
-@pytest.fixture(params=TEST_PAIRS, scope='module')
+@pytest.fixture(params=TEST_PAIRS, scope="module")
 def aggregate_cmp(request, seed=100):
     test_pair = request.param
-    if test_pair == 'np/py':
+    if test_pair == "np/py":
         # Some functions in purepy are not implemented
-        func_ref = _wrap_notimplemented_xfail(aggregate_purepy.aggregate)
+        func_ref = _wrap_notimplemented_skip(aggregate_purepy.aggregate)
         func = aggregate_numpy.aggregate
         group_cnt = 100
     else:
         group_cnt = 1000
         func_ref = aggregate_numpy.aggregate
-        if 'ufunc' in request.param:
+        if "ufunc" in request.param:
             impl = aggregate_numpy_ufunc
-        elif 'numba' in request.param:
+        elif "numba" in request.param:
             impl = aggregate_numba
-        elif 'weave' in request.param:
-            impl = aggregate_weave
-        elif 'pandas' in request.param:
+        elif "pandas" in request.param:
             impl = aggregate_pandas
         else:
             impl = None
@@ -48,7 +51,7 @@ def aggregate_cmp(request, seed=100):
         if not impl:
             pytest.skip("Implementation not available")
         name = _impl_name(impl)
-        func = _wrap_notimplemented_xfail(impl.aggregate, 'aggregate_' + name)
+        func = _wrap_notimplemented_skip(impl.aggregate, "aggregate_" + name)
 
     rnd = np.random.RandomState(seed=seed)
 
@@ -60,7 +63,7 @@ def aggregate_cmp(request, seed=100):
     a = rnd.randn(group_idx.size)
     nana = a.copy()
     nana[::3] = np.nan
-    nana[:(len(nana) // 2)] = np.nan
+    nana[: (len(nana) // 2)] = np.nan
     somea = a.copy()
     somea[somea < 0.3] = 0
     somea[::31] = np.nan
@@ -70,13 +73,13 @@ def aggregate_cmp(request, seed=100):
 def _deselect_purepy(aggregate_cmp, *args, **kwargs):
     # purepy implementation does not handle ndim arrays
     # This is a won't fix and should be deselected instead of skipped
-    return aggregate_cmp.endswith('py')
+    return aggregate_cmp.endswith("py")
 
 
 def _deselect_purepy_nanfuncs(aggregate_cmp, func, *args, **kwargs):
     # purepy implementation does not handle nan values correctly
     # This is a won't fix and should be deselected instead of skipped
-    return 'nan' in getattr(func, '__name__', func) and aggregate_cmp.endswith('py')
+    return "nan" in getattr(func, "__name__", func) and aggregate_cmp.endswith("py")
 
 
 def func_arbitrary(iterator):
@@ -89,16 +92,16 @@ def func_arbitrary(iterator):
 def func_preserve_order(iterator):
     tmp = 0
     for i, x in enumerate(iterator, 1):
-        tmp += x ** i
+        tmp += x**i
     return tmp
 
 
 @pytest.mark.filterwarnings("ignore:numpy.ufunc size changed")
 @pytest.mark.deselect_if(func=_deselect_purepy_nanfuncs)
 @pytest.mark.parametrize("fill_value", [0, 1, np.nan])
-@pytest.mark.parametrize("func", func_list, ids=lambda x: getattr(x, '__name__', x))
+@pytest.mark.parametrize("func", func_list, ids=lambda x: getattr(x, "__name__", x))
 def test_cmp(aggregate_cmp, func, fill_value, decimal=10):
-    is_nanfunc = 'nan' in getattr(func, '__name__', func)
+    is_nanfunc = "nan" in getattr(func, "__name__", func)
     a = aggregate_cmp.nana if is_nanfunc else aggregate_cmp.a
     try:
         ref = aggregate_cmp.func_ref(aggregate_cmp.group_idx, a, func=func, fill_value=fill_value)
@@ -109,8 +112,10 @@ def test_cmp(aggregate_cmp, func, fill_value, decimal=10):
         try:
             res = aggregate_cmp.func(aggregate_cmp.group_idx, a, func=func, fill_value=fill_value)
         except ValueError:
-            if np.isnan(fill_value) and aggregate_cmp.test_pair.endswith('py'):
-                pytest.xfail("pure python version uses lists and does not raise ValueErrors when inserting nan into integers")
+            if np.isnan(fill_value) and aggregate_cmp.test_pair.endswith("py"):
+                pytest.skip(
+                    "pure python version uses lists and does not raise ValueErrors when inserting nan into integers"
+                )
             else:
                 raise
         if isinstance(ref, np.ndarray):
@@ -118,8 +123,8 @@ def test_cmp(aggregate_cmp, func, fill_value, decimal=10):
         try:
             np.testing.assert_allclose(res, ref, rtol=10**-decimal)
         except AssertionError:
-            if 'arg' in func and aggregate_cmp.test_pair.startswith('pandas'):
-                pytest.xfail("pandas doesn't fill indices for all-nan groups with fill_value, but with -inf instead")
+            if "arg" in func and aggregate_cmp.test_pair.startswith("pandas"):
+                pytest.skip("pandas doesn't fill indices for all-nan groups with fill_value, but with -inf instead")
             else:
                 raise
 
@@ -127,14 +132,14 @@ def test_cmp(aggregate_cmp, func, fill_value, decimal=10):
 @pytest.mark.deselect_if(func=_deselect_purepy)
 @pytest.mark.parametrize(["ndim", "order"], product([2, 3], ["C", "F"]))
 def test_cmp_ndim(aggregate_cmp, ndim, order, outsize=100, decimal=14):
-    nindices = int(outsize ** ndim)
+    nindices = int(outsize**ndim)
     outshape = tuple([outsize] * ndim)
     group_idx = np.random.randint(0, outsize, size=(ndim, nindices))
     a = np.random.random(group_idx.shape[1])
 
     res = aggregate_cmp.func(group_idx, a, size=outshape, order=order)
     ref = aggregate_cmp.func_ref(group_idx, a, size=outshape, order=order)
-    if ndim > 1 and order == 'F':
+    if ndim > 1 and order == "F":
         # 1d arrays always return False here
         assert np.isfortran(res)
     else:
