@@ -1,6 +1,7 @@
 import numpy as np
 
-from ..utils_numpy import check_dtype, unpack
+from ..utils import check_dtype, unpack
+
 
 def test_check_dtype():
     dtype = check_dtype(None, "mean", np.arange(10, dtype=int), 10)
